////////////////////////////////////////////////////////////////////////////
//
// Audio LevelMeter.
//
////////////////////////////////////////////////////////////////////////////

#include <16f88.H>
#device ADC=8

#include "led_table.h"

//#define _SERIAL_DEBUG_
#undef _SERIAL_DEBUG_

#USE FAST_IO(A)
#USE FAST_IO(B)

//#fuses HS, NOWDT, PUT, NOPROTECT, NOMCLR, NOBROWNOUT
#fuses HS, NOLVP, NOWDT, PUT, NOPROTECT, NOBROWNOUT, NOMCLR, CCPB3

#use delay(clock=20000000)

#ifdef _SERIAL_DEBUG_
#use rs232(BAUD=9600, XMIT=PIN_A2)
#endif //_SERIAL_DEBUG_

struct tagPORTA{
	short bit0;
	short bit1;
	short bit2;
	short bit3;
	short bit4;
	short bit5;
	short bit6;
	short bit7;
}PORTA;
#byte PORTA = 5

struct tagPORTC{
	short bit0;
	short bit1;
	short bit2;
	short bit3;
	short bit4;
	short bit5;
	short bit6;
	short bit7;
}PORTB;
#byte PORTB = 6

typedef unsigned int1	BIT;
typedef unsigned int16	WORD;
typedef unsigned int32	DWORD;

#define HIGH	1
#define LOW		0

#define PRINT_MODE		PORTA.bit4	//\[h(o[\/hbg\).
#define MOVE_MODE		PORTA.bit5	//\[h(ω/ΐω).
#define LED_NUM1		PORTB.bit1	//LEDݒ1.
#define LED_NUM2		PORTB.bit0	//LEDݒ2.

#define SER_L	PORTB.bit5
#define SER_R	PORTB.bit4
#define SCK		PORTB.bit3
#define RCK		PORTB.bit2

BYTE bMaxLedNum = 16;

//#byte ADRESH = 0x1e
//#byte ADRESL = 0x9e

////////////////////////////////////////////////////////////////////////////
#ifdef _SERIAL_DEBUG_
VOID send_byte(WORD val)
{
	putc((val / 10000) + '0');
	val %= 10000;

	putc((val / 1000) + '0');
	val %= 1000;

	putc((val / 100) + '0');
	val %= 100;

	putc((val / 10) + '0');
	val %= 10;

	putc(val + '0');
}
#endif //_SERIAL_DEBUG_

////////////////////////////////////////////////////////////////////////////
void StartupLED(void)
{
	BYTE i;
	BYTE j;
	BYTE bLedPos;

	//LEDS.
	for(j = 0; j < 128; j++){
		SCK = LOW;

		SER_L = SER_R = LOW;

		SCK = HIGH;
	}
	RCK = HIGH;
	RCK = LOW;

	delay_ms(500);

	bLedPos = 128;
	for(i = 0; i < bMaxLedNum; i++){
		for(j = 1; j <= 128; j++){
			SCK = LOW;

			if(j == bLedPos){
				SER_L = SER_R = HIGH;
			}
			else{
				SER_L = SER_R = LOW;
			}

			SCK = HIGH;
		}
		RCK = HIGH;
		RCK = LOW;

		delay_ms(5);

		bLedPos--;
	}
	delay_ms(500);

	for(i = 0; i < bMaxLedNum; i++){
		bLedPos++;

		for(j = 1; j <= 128; j++){
			SCK = LOW;

			if(j == bLedPos){
				SER_L = SER_R = HIGH;
			}
			else{
				SER_L = SER_R = LOW;
			}

			SCK = HIGH;
		}
		RCK = HIGH;
		RCK = LOW;

		delay_ms(5);
	}
	delay_ms(500);
}

////////////////////////////////////////////////////////////////////////////
// C.
//
// IN:
//
// OUT:
//
// RETURN:
//
////////////////////////////////////////////////////////////////////////////
main()
{
	BYTE i;
	BYTE j;
	BYTE bADCVal_R;
	BYTE bADCVal_L;
	BYTE bCnt_R;
	BYTE bCnt_L;

	//낢돉.
	set_tris_a(0b00111011);
	set_tris_b(0b00000011);

	setup_comparator(NC_NC_NC_NC);
	setup_vref(FALSE);
	//setup_adc_ports(sAN0 | sAN1 | VSS_VDD);
	setup_adc_ports(sAN0 | sAN1 | VSS_VREF);
	setup_adc(ADC_CLOCK_DIV_32);

	setup_spi(SPI_SS_DISABLED); 

	//VtgWX^̃s.
	SER_L = LOW;
	SER_R = LOW;
	SCK = LOW;
	RCK = LOW;

	//LED̓ǂݍ.
	if((LED_NUM2 == HIGH) && (LED_NUM1 == HIGH)){
		bMaxLedNum = 16;
	}
	else if((LED_NUM2 == HIGH) && (LED_NUM1 == LOW)){
		bMaxLedNum = 32;
	}
	else if((LED_NUM2 == LOW) && (LED_NUM1 == HIGH)){
		bMaxLedNum = 64;
	}
	else if((LED_NUM2 == LOW) && (LED_NUM1 == LOW)){
		bMaxLedNum = 128;
	}

	//NLEDmF.
	StartupLED();

	//C[v.
	while(1){
		set_adc_channel(0);			//ADϊJn.
		delay_us(15);
		bADCVal_R = read_adc();	//ADϊl̓ǂݍ.

		#ifdef _SERIAL_DEBUG_
		send_byte(bADCVal_R); putc('\r'); putc('\n');
		#endif

		set_adc_channel(1);			//ADϊJn.
		delay_us(15);
		bADCVal_L = read_adc();	//ADϊl̓ǂݍ.

		switch(bMaxLedNum){
		case 16:
			{
				if(MOVE_MODE == HIGH){	//ΐω[h.
					bCnt_R = LogTable16[bADCVal_R];
					bCnt_L = LogTable16[bADCVal_L];
				}
				else{					//jAω[h.
					bCnt_R = LinearTable16[bADCVal_R];
					bCnt_L = LinearTable16[bADCVal_L];
				}
			}
			break;
		case 32:
			{
				if(MOVE_MODE == HIGH){	//ΐω[h.
					bCnt_R = LogTable32[bADCVal_R];
					bCnt_L = LogTable32[bADCVal_L];
				}
				else{					//jAω[h.
					bCnt_R = LinearTable32[bADCVal_R];
					bCnt_L = LinearTable32[bADCVal_L];
				}
			}
			break;
		case 64:
			{
				if(MOVE_MODE == HIGH){	//ΐω[h.
					bCnt_R = LogTable64[bADCVal_R];
					bCnt_L = LogTable64[bADCVal_L];
				}
				else{					//jAω[h.
					bCnt_R = LinearTable64[bADCVal_R];
					bCnt_L = LinearTable64[bADCVal_L];
				}
			}
			break;
		case 128:
			{
				if(MOVE_MODE == HIGH){	//ΐω[h.
					bCnt_R = LogTable128[bADCVal_R];
					bCnt_L = LogTable128[bADCVal_L];
				}
				else{					//jAω[h.
					bCnt_R = LinearTable128[bADCVal_R];
					bCnt_L = LinearTable128[bADCVal_L];
				}
			}
			break;
		}

		RCK = LOW;

		if(PRINT_MODE == HIGH){	//o[\.
			for(i = 0; i < bMaxLedNum; i++){
				SCK = LOW;

				if(bCnt_R <= i){
					SER_R = 1;
				}
				else{
					SER_R = 0;
				}

				if(bCnt_L <= i){
					SER_L = 1;
				}
				else{
					SER_L = 0;
				}

				SCK = HIGH;
			}
		}
		else{
			for(i = 0; i < bMaxLedNum; i++){
				SCK = LOW;

				if(bCnt_R == i){
					SER_R = 1;
				}
				else{
					SER_R = 0;
				}

				if(bCnt_L == i){
					SER_L = 1;
				}
				else{
					SER_L = 0;
				}

				SCK = HIGH;
			}
		}

		RCK = HIGH;
	}
}

